#include "RenderWindow.hpp"

#include "SFML/Window/WindowStyle.hpp"


extern PyTypeObject PySfEventType;
extern PyTypeObject PySfViewType;
extern PyTypeObject PySfColorType;
extern PyTypeObject PySfWindowType;

static PyMemberDef PySfRenderWindow_members[] = {
	{NULL}  /* Sentinel */
};


static void
PySfRenderWindow_dealloc(PySfRenderWindow* self)
{
	delete self->obj;
	self->ob_type->tp_free((PyObject*)self);
}

static PyObject *
PySfRenderWindow_new(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
	PySfRenderWindow *self;

	self = (PySfRenderWindow *)type->tp_alloc(type, 0);
	if (self != NULL)
	{
	}

	return (PyObject *)self;
}


static PyObject*
PySfRenderWindow_GetEvent(PySfRenderWindow* self, PyObject *args)
{
	PySfEvent *PyEvent = (PySfEvent *)args;

	if (! PyObject_TypeCheck(PyEvent, &PySfEventType))
	{
		PyErr_SetString(PyExc_TypeError, "Argument is not a sfEvent");
		return NULL;
	}

	if (self->obj->GetEvent(*(PyEvent->obj)))
	{
		PyEvent->Type = PyEvent->obj->Type;
		PyEvent->Text->Unicode = PyEvent->obj->Text.Unicode;
		PyEvent->Key->Code = PyEvent->obj->Key.Code;
		if (PyEvent->obj->Key.Alt)
			PyEvent->Key->Alt = Py_True;
		else
			PyEvent->Key->Alt = Py_False;
		if (PyEvent->obj->Key.Control)
			PyEvent->Key->Control = Py_True;
		else
			PyEvent->Key->Control = Py_False;
		if (PyEvent->obj->Key.Shift)
			PyEvent->Key->Shift = Py_True;
		else
			PyEvent->Key->Shift = Py_False;
		PyEvent->MouseButton->Button = PyEvent->obj->MouseButton.Button;
		PyEvent->MouseMove->X = PyEvent->obj->MouseMove.X;
		PyEvent->MouseMove->Y = PyEvent->obj->MouseMove.Y;
		PyEvent->JoyMove->JoystickId = PyEvent->obj->JoyMove.JoystickId;
		PyEvent->JoyButton->JoystickId = PyEvent->obj->JoyButton.JoystickId;
		PyEvent->JoyButton->Button = PyEvent->obj->JoyButton.Button;
		PyEvent->JoyMove->Axis = PyEvent->obj->JoyMove.Axis;
		PyEvent->JoyMove->Position = PyEvent->obj->JoyMove.Position;
		PyEvent->Size->Width = PyEvent->obj->Size.Width;
		PyEvent->Size->Height = PyEvent->obj->Size.Height;
		PyEvent->MouseWheel->Delta = PyEvent->obj->MouseWheel.Delta;
		Py_RETURN_TRUE;
	}
	else
		Py_RETURN_FALSE;
}


static PyObject*
PySfRenderWindow_Create(PySfRenderWindow* self, PyObject *args, PyObject *kwds)
{
	PyObject *VideoModeTmp=NULL;
	sf::VideoMode *VideoMode;
	char *Title=NULL;
	unsigned long WindowStyle = sf::Style::Resize | sf::Style::Close;
	int AntialiasingLevel=0;

	char *kwlist[] = {"VideoMode", "Title", "WindowStyle", "AntialiasingLevel", NULL};

	if (! PyArg_ParseTupleAndKeywords(args, kwds, "Os|Ii", kwlist, &VideoModeTmp, &Title, &WindowStyle, &AntialiasingLevel))
		return NULL; 

	if (VideoModeTmp) {
		VideoMode = ((PySfVideoMode *)VideoModeTmp)->obj;
		PySfVideoModeUpdate((PySfVideoMode *)VideoModeTmp);
	}
	else
		return NULL;

	self->obj->Create(*VideoMode, Title, WindowStyle, AntialiasingLevel);
	// self->obj->Create(*VideoMode, Title);

	Py_RETURN_NONE;
}


static int
PySfRenderWindow_init(PySfRenderWindow *self, PyObject *args, PyObject *kwds)
{
	self->obj = new sf::RenderWindow();
	PySfRenderWindow_Create(self, args, kwds);
	return 0;
}


static PyObject *
PySfRenderWindow_Display(PySfRenderWindow *self)
{
	self->obj->Display();
	Py_RETURN_NONE;
}


static PyObject *
PySfRenderWindow_Draw(PySfRenderWindow *self, PyObject *args)
{
	PySfDrawable *Drawable = (PySfDrawable *)args;

	if (!args)
		return NULL;

	self->obj->Draw( *(Drawable->obj) );
	Py_RETURN_NONE;
}

static PyObject *
PySfRenderWindow_Capture(PySfRenderWindow *self)
{
	PySfImage *Image;

	Image = GetNewPySfImage();
	Image->obj = new sf::Image(self->obj->Capture());
	return (PyObject *)Image;
}

static PyObject *
PySfRenderWindow_SetBackgroundColor(PySfRenderWindow* self, PyObject *args)
{
	PySfColor *Color = (PySfColor *)args;
	if (! PyObject_TypeCheck(Color, &PySfColorType))
	{
		PyErr_SetString(PyExc_TypeError, "Argument is not a sfColor");
		return NULL;
	}
	PySfColorUpdate(Color);
	self->obj->SetBackgroundColor(*(Color->obj));
	Py_RETURN_NONE;
}

static PyObject *
PySfRenderWindow_GetViewRect(PySfRenderWindow *self)
{
	PySfFloatRect *Rect;

	Rect = GetNewPySfFloatRect();
	Rect->obj = new sf::FloatRect ( self->obj->GetViewRect() );
	Rect->Left = Rect->obj->Left;
	Rect->Top = Rect->obj->Top;
	Rect->Right = Rect->obj->Right;
	Rect->Bottom = Rect->obj->Bottom;

	return (PyObject *)Rect;
}

static PyObject *
PySfRenderWindow_SetView(PySfRenderWindow* self, PyObject *args)
{
	PySfView *View = (PySfView *)args;
	if (! PyObject_TypeCheck(View, &PySfViewType))
	{
		PyErr_SetString(PyExc_TypeError, "Argument is not a sfView");
		return NULL;
	}
	PySfViewUpdate(View);
	self->obj->SetView( (View->obj));
	Py_RETURN_NONE;
}

static PyObject *
PySfRenderWindow_OptimizeForNonOpenGL(PySfRenderWindow *self, PyObject *args)
{
	bool Optimize = true;
	if (args == Py_False)
		Optimize = false;
	self->obj->OptimizeForNonOpenGL(Optimize);
	Py_RETURN_NONE;
}

static PyMethodDef PySfRenderWindow_methods[] = {
	{"Create", (PyCFunction)PySfRenderWindow_Create, METH_VARARGS | METH_KEYWORDS, "Create a window"},
	{"Display", (PyCFunction)PySfRenderWindow_Display, METH_NOARGS, "Display the window on screen."},
	{"Draw", (PyCFunction)PySfRenderWindow_Draw, METH_O, "Draw something on the window."},
	{"GetEvent", (PyCFunction)PySfRenderWindow_GetEvent, METH_O, "Get the event on top of events stack, if any, and pop it."},
	{"Capture", (PyCFunction)PySfRenderWindow_Capture, METH_NOARGS, "Save the content of the window to an image."},
	{"SetBackgroundColor", (PyCFunction)PySfRenderWindow_SetBackgroundColor, METH_O, "Change the background color of the window."},
	{"GetViewRect", (PyCFunction)PySfRenderWindow_GetViewRect, METH_NOARGS, "Get the current view rectangle."},
	{"SetView", (PyCFunction)PySfRenderWindow_SetView, METH_O, "Change the current active view."},
	{"OptimizeForNonOpenGL", (PyCFunction)PySfRenderWindow_OptimizeForNonOpenGL, METH_O, "Tell SFML to optimize its calls to the graphics driver, in case the user is not doing custom OpenGL calls."},
	{"GetEvent", (PyCFunction)PySfRenderWindow_GetEvent, METH_O, "Get the event on top of events stack, if any, and pop it."},
	{NULL}  /* Sentinel */
};

PyTypeObject PySfRenderWindowType = {
	PyObject_HEAD_INIT(NULL)
	0,                         /*ob_size*/
	"RenderWindow",             /*tp_name*/
	sizeof(PySfRenderWindow),             /*tp_basicsize*/
	0,                         /*tp_itemsize*/
	(destructor)PySfRenderWindow_dealloc, /*tp_dealloc*/
	0,                         /*tp_print*/
	0,                         /*tp_getattr*/
	0,                         /*tp_setattr*/
	0,                         /*tp_compare*/
	0,                         /*tp_repr*/
	0,                         /*tp_as_number*/
	0,                         /*tp_as_sequence*/
	0,                         /*tp_as_mapping*/
	0,                         /*tp_hash */
	0,                         /*tp_call*/
	0,                         /*tp_str*/
	0,                         /*tp_getattro*/
	0,                         /*tp_setattro*/
	0,                         /*tp_as_buffer*/
	Py_TPFLAGS_DEFAULT, /*tp_flags*/
	"Simple wrapper for sfWindow that allows easy 2D rendering.",           /* tp_doc */
	0,		               /* tp_traverse */
	0,		               /* tp_clear */
	0,		               /* tp_richcompare */
	0,		               /* tp_weaklistoffset */
	0,		               /* tp_iter */
	0,		               /* tp_iternext */
	PySfRenderWindow_methods,             /* tp_methods */
	PySfRenderWindow_members,             /* tp_members */
	0,                         /* tp_getset */
	&PySfWindowType,                         /* tp_base */
	0,                         /* tp_dict */
	0,                         /* tp_descr_get */
	0,                         /* tp_descr_set */
	0,                         /* tp_dictoffset */
	(initproc)PySfRenderWindow_init,      /* tp_init */
	0,                         /* tp_alloc */
	PySfRenderWindow_new,                 /* tp_new */
};


