#include "View.hpp"

extern PyTypeObject PySfFloatRectType;

static PyMemberDef PySfView_members[] = {
	{"Rect", T_OBJECT, offsetof(PySfView, Rect), 0, "View Rect."},
	{"Zoom", T_FLOAT, offsetof(PySfView, Zoom), 0, "Zoom (less than 1 is unzoom, greater than 1 is zoom)."},
	{NULL}  /* Sentinel */
};


static void
PySfView_dealloc(PySfView *self)
{
	delete self->obj;
	self->ob_type->tp_free((PyObject*)self);
}

static PyObject *
PySfView_new(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
	PySfView *self;

	self = (PySfView *)type->tp_alloc(type, 0);
	if (self != NULL)
	{
		self->Rect = NULL;
		// self->Rect = PyObject_New(PySfFloatRect, &PySfFloatRectType);
		self->Zoom = 1;
	}

	return (PyObject *)self;
}

void
PySfViewUpdate(PySfView *self)
{
	PySfFloatRectUpdate(self->Rect);
	self->obj->Rect.Left = self->Rect->obj->Left;
	self->obj->Rect.Top = self->Rect->obj->Top;
	self->obj->Rect.Right = self->Rect->obj->Right;
	self->obj->Rect.Bottom = self->Rect->obj->Bottom;
	self->obj->Zoom = self->Zoom;
}

static int
PySfView_init(PySfView *self, PyObject *args, PyObject *kwds)
{
	if (! PyArg_ParseTuple(args, "|O!f", &PySfFloatRectType, &self->Rect, &self->Zoom))
		return -1;

	if (self->Rect != NULL)
		self->obj = new sf::View( (const sf::FloatRect&) self->Rect->obj, self->Zoom);
	else
		self->obj = new sf::View();

	return 0;
}

static PyMethodDef PySfView_methods[] = {
	{NULL}  /* Sentinel */
};

PyTypeObject PySfViewType = {
	PyObject_HEAD_INIT(NULL)
	0,                         /*ob_size*/
	"View",             /*tp_name*/
	sizeof(PySfView),             /*tp_basicsize*/
	0,                         /*tp_itemsize*/
	(destructor)PySfView_dealloc, /*tp_dealloc*/
	0,                         /*tp_print*/
	0,                         /*tp_getattr*/
	0,                         /*tp_setattr*/
	0,                         /*tp_compare*/
	0,                         /*tp_repr*/
	0,                         /*tp_as_number*/
	0,                         /*tp_as_sequence*/
	0,                         /*tp_as_mapping*/
	0,                         /*tp_hash */
	0,                         /*tp_call*/
	0,                         /*tp_str*/
	0,                         /*tp_getattro*/
	0,                         /*tp_setattro*/
	0,                         /*tp_as_buffer*/
	Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE, /*tp_flags*/
	"sfView defines a view (position, size and zoom) ; you can consider it as a camera.",           /* tp_doc */
	0,		               /* tp_traverse */
	0,		               /* tp_clear */
	0,		               /* tp_richcompare */
	0,		               /* tp_weaklistoffset */
	0,		               /* tp_iter */
	0,		               /* tp_iternext */
	PySfView_methods,             /* tp_methods */
	PySfView_members,             /* tp_members */
	0,                         /* tp_getset */
	0,                         /* tp_base */
	0,                         /* tp_dict */
	0,                         /* tp_descr_get */
	0,                         /* tp_descr_set */
	0,                         /* tp_dictoffset */
	(initproc)PySfView_init,      /* tp_init */
	0,                         /* tp_alloc */
	PySfView_new,                 /* tp_new */
};


